@@ -3,7 +3,9 @@
 import base64
 import datetime
 from getpass import getpass
+import hashlib
 import json
+import inspect
 import logging
 import os
 import platform
@@ -20,6 +22,8 @@ from onepasswordconnectsdkopexe.models import OPItem as Item
 from onepasswordconnectsdkopexe.models import OPVault as Vault
 import onepasswordconnectsdkopexe.models
 
+logger = logging.getLogger('onepasswordconnectsdkopexe')
+
 
 class Client:
 
@@ -48,7 +52,7 @@ class Client:
 
     """Python Client Class"""
 
-    def __init__(self, url: str, email_address: str, secret_key: str, master_password: str, account: str, exe: str=None):
+    def __init__(self, url: str, email_address: str, secret_key: str, master_password: str, account: str= '', exe: str=None):
         """Initialize client"""
         if not exe:
             if getattr(sys, 'frozen', False):
@@ -62,9 +66,10 @@ class Client:
             self.exe = exe
         if not os.path.isfile(self.exe):
             raise RuntimeError('the file "{}" cannot be found'.format(self.exe))
-        self.account = account
+         
         self.url = url
         self.email_address = email_address
+        self.account = account if account else hashlib.sha256(f'{self.email_address}|{self.url}').hexdigest()
         self.secret_key = secret_key
 
         self.master_password = master_password
@@ -73,6 +78,7 @@ class Client:
         self.global_flags = self._get_global_flags()
 
     def create_token(self):
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         cmd = '"{exe}" signin "{url}" "{email_address}" "{secret_key}" --raw --shorthand="{account}"'.format(**self.__dict__)
         master_password = self.master_password if isinstance(self.master_password, bytes) else self.master_password.encode()
         rc, stdout, stderr = self._process_raise(cmd=cmd,stdin=master_password)
@@ -88,6 +94,7 @@ class Client:
         Returns:
         Item object: The found item
         """
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         cmd = '"{exe}" get item "{item_id}" {global_flags} --vault "{vault_id}"'.format(**self.__dict__, item_id=item_id, vault_id=vault_id)
         rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
         data = json.loads(stdout.strip())
@@ -103,10 +110,10 @@ class Client:
         Returns:
         Item object: A summary of the found item
         """
-
+        logger.debug(f'{inspect.stack()[0][3]} : start')
 
         cmd = '"{exe}" list items  {global_flags} --vault "{vault_id}"'.format(**self.__dict__, vault_id=vault_id)
-        rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
+        rc, stdout, stderr = self._process_raise(cmd=cmd, tail_stdout=0)
         items = json.loads(stdout.strip())
         items_fond = []
         for i, item in enumerate(items):
@@ -119,8 +126,9 @@ class Client:
                 f"Found {len(items_fond)} items in vault {vault_id} with \
                     title {title}"
             )
-
-        return self.__deserialize(items_fond, "list[SummaryItem]")[0]
+        
+        return self.get_item(items_fond[0]['uuid'], vault_id)
+#         return self.__deserialize(items_fond, "list[OPSummaryItem]")[0]
 
     def get_items(self, vault_id: str):
         """Returns a list of item summaries for the specified vault
@@ -133,18 +141,18 @@ class Client:
             from the 1Password Connect API
 
         Returns:
-            List[SummaryItem]: A list of summarized items
+            List[OPSummaryItem]: A list of summarized items
         """
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         cmd = '"{exe}" list items  {global_flags} --vault "{vault_id}"'.format(**self.__dict__, vault_id=vault_id)
         rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
         items = json.loads(stdout.strip())
         for i, item in enumerate(items):
             items[i]=self.__organise_item(item)
 
-        return self.__deserialize(items, "list[SummaryItem]")
+        return self.__deserialize(items, "list[OPSummaryItem]")
 
     def __organise_item(self, item):
-
         if 'overview' in item:
             item = {**item, **item['overview']}
             del item['overview']
@@ -167,12 +175,13 @@ class Client:
             type_field = {'T': 'STRING', 'P': 'CONCEALED'}
             for i, old_field in enumerate(item['fields']):
                 field = {
+                    "id": old_field['designation'],
                     "type": type_field[ old_field['type']],
                     "purpose": old_field['designation'].upper(),
                     "label": old_field['name'],
                     "value": old_field['value']
                 }
-                if 'id' in old_field:
+                if 'id' in old_field and old_field['id']:
                     field["id"] = old_field['id']
                 item['fields'][i] = field
         else:
@@ -234,9 +243,10 @@ class Client:
             FailedToRetrieveItemException: Thrown when a HTTP error is returned
             from the 1Password Connect API
         """
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         cmd = '"{exe}" delete  item "{item_id}" {global_flags} --vault "{vault_id}"'.format(**self.__dict__, item_id=item_id, vault_id=vault_id)
         rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
-        stdout
+        return stdout
 
     def create_item(self, vault_id: str, item: Item):
         """Creates an item at the specified vault
@@ -252,44 +262,55 @@ class Client:
         Returns:
             Item: The created item
         """
+        logger.debug(f'{inspect.stack()[0][3]} : start')
 
         serialized_body = self.sanitize_for_serialization(item)
+        
+        logger.debug(f'serialized_body : {serialized_body}')
 
-        primary_url = ""
-        if 'URLs' in serialized_body:
-            for i, url in enumerate(serialized_body['URLs']):
-                if 'primary' in url:
-                    if url['primary'] and not primary_url:
-                        primary_url = url['u']
-                    del serialized_body['URLs'][i]['primary']
-
-        if primary_url and len(serialized_body['URLs']):
-            primary_url = serialized_body['URLs'][0]['u']
+        primary_url = self._get_primary_url(serialized_body['URLs']) if 'URLs' in serialized_body else ""
 
         if not( 'templateUuid' in serialized_body and serialized_body['templateUuid']):
             serialized_body['templateUuid'] = 'LOGIN'
 
         encoded_item = {}
+
+        self._create_item_field(encoded_item, serialized_body['fields'], serialized_body['templateUuid'])
+        for section in serialized_body['sections']:
+            if section['id'] in encoded_item['sections']:
+                encoded_item['sections'][section['id']]['title'] = section['label']
+            else:
+                encoded_item['sections'][section['id']] = {'title':section['label']}
+            encoded_item['sections'][section['id']]['name']= f'Section_{str(uuid.uuid4())}'
+
+        encoded_item['sections'] = list(encoded_item['sections'].values())
+
+        cmd = '"{exe}" encode {global_flags}'.format(**self.__dict__)
+
+        logger.debug(f'data send : {encoded_item}')
+        rc, stdout, stderr = self._process_raise(cmd=cmd,stdin=json.dumps(encoded_item).encode())
+        encoded_item = stdout.strip()
+
+        cmd = '"{exe}" create item "{category}" "{encoded_item}" {global_flags} --vault "{vault_id}"'.format(**self.__dict__, category=serialized_body['templateUuid'].lower(), encoded_item=encoded_item, vault_id=vault_id)
+        if 'title' in serialized_body:
+            cmd = f'{cmd} --title "{self._escape_quote(serialized_body["title"])}"'
+        if primary_url:
+            cmd = f'{cmd} --url "{self._escape_quote(primary_url)}"'
+
+        logger
+        if 'tags' in serialized_body and serialized_body['tags']:
+            cmd = '{cmd} --tags "{tags}"'.format(cmd=cmd, tags=self._escape_quote(",".join(serialized_body['tags'])))
+
+        rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
+        return self.deserialize(stdout.strip(), "OPItem")
+
+    def _create_item_field(self, encoded_item: dict, fields: dict, template_uuid: str):
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         encoded_item['fields'] = []
         encoded_item['sections'] = {}
-#         encoded_item['URLs'] = serialized_body['URLs']   # Not supported
-
-        type_field = {'STRING': 'T', 'CONCEALED': 'P'}
-        for field in serialized_body['fields']:
+        for field in fields:
             if 'purpose' in field:
-                if field['purpose'] == 'NOTES':
-                    encoded_item['notesPlain'] = field['value']
-                elif field['purpose'] == 'PASSWORD' and serialized_body['templateUuid'] == '005':
-                    encoded_item['password'] = field['value']
-                else:
-                    encoded_field = {
-                        'id': "",
-                        'designation': field['purpose'].lower(),
-                        'name': field['label'] ,
-                        'type': type_field[field['type']],
-                        'value': field['value']
-                        }
-                    encoded_item['fields'].append(encoded_field)
+                self._create_item_field_purpose(encoded_item, field, template_uuid )
             elif 'section' in field:
                 if not 'id' in field['section']:
                     continue
@@ -302,36 +323,36 @@ class Client:
                 if not field['section']['id'] in encoded_item['sections']:
                     encoded_item['sections'][field['section']['id']] = {'fields':[]}
                 encoded_item['sections'][field['section']['id']]['fields'].append(encoded_field)
+    
+    def _create_item_field_purpose(self, encoded_item: dict, field: dict, template_uuid: str):
+        logger.debug(f'{inspect.stack()[0][3]} : start')
+        type_field = {'STRING': 'T', 'CONCEALED': 'P'}
+        if field['purpose'] == 'NOTES':
+            encoded_item['notesPlain'] = field['value']
+        elif field['purpose'] == 'PASSWORD' and template_uuid == '005':
+            encoded_item['password'] = field['value']
+        else:
+            encoded_field = {
+                'id': "",
+                'designation': field['purpose'].lower(),
+                'name': field['label'] ,
+                'type': type_field[field['type']],
+                'value': field['value']
+                }
+            encoded_item['fields'].append(encoded_field)
+            
+    def _get_primary_url(self, urls: dict) -> str:
+        logger.debug(f'{inspect.stack()[0][3]} : start')
+        primary_url = ""
+        for i, url in enumerate(urls):
+            if 'primary' in url:
+                if url['primary'] and not primary_url:
+                    primary_url = url['u']
+                del urls[i]['primary']
 
-        for section in serialized_body['sections']:
-            if section['id'] in encoded_item['sections']:
-                encoded_item['sections'][section['id']]['title'] = section['label']
-            else:
-                encoded_item['sections'][section['id']] = {'title':section['label']}
-            encoded_item['sections'][section['id']]['name']= f'Section_{str(uuid.uuid4())}'
-
-        encoded_item['sections'] = list(encoded_item['sections'].values())
-
-        cmd = '"{exe}" encode {global_flags}'.format(**self.__dict__)
-
-        rc, stdout, stderr = self._process_raise(cmd=cmd,stdin=json.dumps(encoded_item).encode())
-        encoded_item = stdout.strip()
-
-        decode_item = json.loads( base64.b64decode(encoded_item.encode() + b'=' * (-len(encoded_item) % 4)).decode('utf-8'))
-
-        cmd = '"{exe}" create item "{category}" "{encoded_item}" {global_flags} --vault "{vault_id}"'.format(**self.__dict__, category=serialized_body['templateUuid'].lower(), encoded_item=encoded_item, vault_id=vault_id)
-        if 'title' in serialized_body:
-            cmd = f'{cmd} --title "{self._escape_quote(serialized_body["title"])}"'
-        if primary_url:
-            cmd = f'{cmd} --url "{self._escape_quote(primary_url)}"'
-
-        if 'tags' in serialized_body and serialized_body['tags']:
-            cmd = '{cmd} --tags "{tags}"'.format(cmd=cmd, tags=self._escape_quote(",".join(serialized_body['tags'])))
-
-        rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
-        return self.deserialize(stdout.strip(), "OPItem")
-
-
+        if not primary_url :
+            primary_url = urls[0]['u']
+        return primary_url
 
     def update_item(self, item_id: str, vault_id: str, item: Item):
         """Update the specified item at the specified vault.
@@ -348,19 +369,11 @@ class Client:
         Returns:
             Item: The updated item
         """
+        logger.debug(f'{inspect.stack()[0][3]} : start')
 
         serialized_body = self.sanitize_for_serialization(item)
 
-        primary_url = ""
-        if 'URLs' in serialized_body:
-            for i, url in enumerate(serialized_body['URLs']):
-                if 'primary' in url:
-                    if url['primary'] and not primary_url:
-                        primary_url = url['u']
-                    del serialized_body['URLs'][i]['primary']
-
-        if primary_url and len(serialized_body['URLs']):
-            primary_url = serialized_body['URLs'][0]['u']
+        primary_url = self._get_primary_url(serialized_body['URLs']) if 'URLs' in serialized_body else ""
 
         assignments = ' '
         if 'title' in serialized_body:
@@ -371,25 +384,38 @@ class Client:
         if 'tags' in serialized_body and serialized_body['tags']:
             assignments += 'tags="{tags}" '.format( tags=self._escape_quote(",".join(serialized_body['tags'])))
 
-        for field in serialized_body['fields']:
+        assignments += self._update_item_field(serialized_body['fields'])
+           
+
+        logger.debug(f'data send : {assignments}')
+        cmd = '"{exe}" edit item "{item_id}" {assignments} {global_flags} --vault "{vault_id}"'.format(**self.__dict__, category=serialized_body['templateUuid'].lower(), item_id=item_id, assignments=assignments, vault_id=vault_id)
+        rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
+
+        return self.get_item( item_id, vault_id)
+
+    def _update_item_field(self, fields)-> str:
+        logger.debug(f'{inspect.stack()[0][3]} : start')
+        assignments: str = ''
+        for field in fields: 
             if 'purpose' in field:
-                if field['purpose'] == 'NOTES':
-                    assignments += 'notesPlain="{value}" '.format( value=self._escape_quote(field["value"]))
-                else:
-                    assignments += '{id}="{value}" '.format(id=field["label"], value=self._escape_quote(field["value"]))
+                assignments += self._update_item_field_purpose(field)
             elif 'section' in field:
                 if not 'id' in field['section']:
                     continue
                 if not field['type'] in ['DATE', 'MONTH_YEAR', 'ADDRESS']:
                     assignments += '{id}="{value}" '.format(id=field["id"], value=self._escape_quote(field["value"]))
                 else:
-                    logging.debug(f'"{field["type"]}" fields aren\'t supported yet')
-
-
-        cmd = '"{exe}" edit item "{item_id}" {assignments} {global_flags} --vault "{vault_id}"'.format(**self.__dict__, category=serialized_body['templateUuid'].lower(), item_id=item_id, assignments=assignments, vault_id=vault_id)
-        rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
-
-        return self.get_item( item_id, vault_id)
+                    logger.debug(f'"{field["type"]}" fields aren\'t supported yet')
+        return assignments
+    
+    def _update_item_field_purpose(self, field)-> str:
+        logger.debug(f'{inspect.stack()[0][3]} : start')
+        assignments = ''
+        if field['purpose'] == 'NOTES':
+            assignments += 'notesPlain="{value}" '.format( value=self._escape_quote(field["value"]))
+        else:
+            assignments += '{id}="{value}" '.format(id=field["label"], value=self._escape_quote(field["value"]))
+        return assignments            
 
     def get_vault(self, vault_id: str):
         """Returns the vault with the given vault_id
@@ -404,7 +430,7 @@ class Client:
         Returns:
             Vault: The specified vault
         """
-
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         cmd = '"{exe}" get vault "{vault_id}" {global_flags}'.format(**self.__dict__, vault_id=vault_id)
         rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
         vault = json.loads(stdout.strip())
@@ -422,7 +448,7 @@ class Client:
         Returns:
             List[Vault]: All vaults for the service account in use
         """
-
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         cmd = '"{exe}" list vaults {global_flags}| "{exe}" get vault - {global_flags}'.format(**self.__dict__)
         rc, stdout, stderr = self._process_raise(cmd=cmd,tail_stdout=0)
         vaults = []
@@ -432,6 +458,7 @@ class Client:
         return self.__deserialize(vaults, "list[OPVault]")
 
     def __organise_vault(self, vault):
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         if 'type' in vault:
             vault['type'] = list(Vault.types.keys())[list(Vault.types.values()).index(vault['type'])] if vault['type'] in list(Vault.types.values()) else None
 
@@ -447,6 +474,7 @@ class Client:
         Returns:
         Response object: The request response
         """
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         url = f"{self.url}{path}"
 
         if body:
@@ -465,6 +493,7 @@ class Client:
 
         :return: deserialized object.
         """
+        logger.debug(f'{inspect.stack()[0][3]} : start')
         # fetch data from response object
         try:
             data = json.loads(response)
@@ -544,7 +573,6 @@ class Client:
                 klass = self.NATIVE_TYPES_MAPPING[klass]
             else:
                 klass = getattr(onepasswordconnectsdkopexe.models, klass)
-                pass
 
         if klass in self.PRIMITIVE_TYPES:
             return self.__deserialize_primitive(data, klass)
@@ -684,7 +712,7 @@ class Client:
         '''
 
         if show_cmd:
-            logging.debug('execute : {} - on : {}'.format(cmd, cwd))
+            logger.debug('execute : {} - on : {}'.format(cmd, cwd))
         proc = subprocess.Popen(cmd,
                                 cwd = cwd,
                                 shell=True,
@@ -696,20 +724,20 @@ class Client:
         stdout,stderr = proc.communicate( input=stdin, timeout=timeout)
         proc.poll()
         returncode = proc.returncode
-        logging.debug('return error code : {}'.format(returncode))
+        logger.debug('return error code : {}'.format(returncode))
         stdout = stdout.decode('utf8').strip()
         stderr = stderr.decode('utf8').strip()
         if stdout:
             if tail_stdout is None:
-                logging.debug('stdout = {}'.format(stdout))
+                logger.debug('stdout = {}'.format(stdout))
             elif tail_stdout > 0:
                 array_stdout = stdout.split("\n")
                 p_stdout = "\n".join(array_stdout[tail_stdout*-1:])
-                logging.debug('stdout = {}'.format(p_stdout))
+                logger.debug('tail stdout = {}'.format(p_stdout))
         if not returncode == 0:
-            logging.error('stderr = {}'.format(stderr))
+            logger.error('stderr = {}'.format(stderr))
         elif stderr:
-            logging.debug('stdout = {}'.format(stderr))
+            logger.debug('stderr = {}'.format(stderr))
         return returncode, stdout, stderr
 
     def _process_raise(self, cmd, cwd=None, tail_stdout=None, show_cmd=True, stdin = None, timeout=15):
